package ca.phon.app.hooks;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;

import ca.phon.plugin.IPluginExtensionPoint;
import ca.phon.plugin.PluginManager;

/**
 * 
 */
public abstract class HookableAction extends AbstractAction {

	private static final long serialVersionUID = -77335181664938946L;

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public final void actionPerformed(ActionEvent ae) {
		final List<IPluginExtensionPoint<ActionHook>> startupHookPts =
				PluginManager.getInstance().getExtensionPoints(ActionHook.class);
		final Class<? extends HookableAction> myClass = getClass();
		boolean doAction = true;
		for(IPluginExtensionPoint<ActionHook> actionHookExt:startupHookPts) {
			final ActionHook<HookableAction> actionHook = actionHookExt.getFactory().createObject();
			if(actionHook.getActionType() == myClass) {
				doAction &= !actionHook.beforeAction(this, ae);
			}
			if(!doAction) break;
		}
		
		if(doAction)
			hookableActionPerformed(ae);
		
		for(IPluginExtensionPoint<ActionHook> actionHookExt:startupHookPts) {
			final ActionHook<HookableAction> actionHook = actionHookExt.getFactory().createObject();
			if(actionHook.getActionType() == myClass) {
				actionHook.afterAction(this, ae);
			}
		}
	}
	
	/**
	 * Method which should be implemented instead of 
	 * actionPerformed.
	 * 
	 * @param action event
	 */
	public abstract void hookableActionPerformed(ActionEvent ae);
	
}
