/*
 * Phon - An open source tool for research in phonology.
 * Copyright (C) 2008 The Phon Project, Memorial University <http://phon.ling.mun.ca>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ca.phon.app.prefs;

import java.awt.BorderLayout;
import java.util.List;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.WindowConstants;

import ca.phon.plugin.IPluginExtensionFactory;
import ca.phon.plugin.IPluginExtensionPoint;
import ca.phon.plugin.PluginManager;
import ca.phon.ui.action.PhonActionEvent;
import ca.phon.ui.action.PhonUIAction;
import ca.phon.ui.decorations.DialogHeader;

import com.jgoodies.forms.builder.ButtonBarBuilder;


/**
 * Dialog for application preferences.
 *
 */
public class PrefsDialog extends JDialog {
	
	private static final long serialVersionUID = 6513355180838862607L;
	
	/**
	 * pref panels
	 */
	private GeneralPrefsPanel generalPrefs;
	private EditorPrefsPanel editorPrefs;
	private MediaPrefsPanel mediaPrefs;
	
	private DialogHeader header;
	
	private JTabbedPane tabbedPane;
	
	private JButton okButton;
	
	/**
	 * 
	 */
	public PrefsDialog() {
		super();
		super.setTitle("Phon : Preferences");
		
		init();
	}
	
	public PrefsDialog(JFrame f) {
		super(f);
		super.setTitle("Phon : Preferences");
		
		init();
	}
	
	private void init() {
		generalPrefs = new GeneralPrefsPanel();
		editorPrefs = new EditorPrefsPanel();
		mediaPrefs = new MediaPrefsPanel();
		
		tabbedPane = new JTabbedPane();
		tabbedPane.addTab(generalPrefs.getTitle(), generalPrefs);
		tabbedPane.addTab(editorPrefs.getTitle(), editorPrefs);
		tabbedPane.addTab(mediaPrefs.getTitle(), mediaPrefs);
		
		List<IPluginExtensionPoint<PrefsPanel>> prefExtPts = 
			PluginManager.getInstance().getExtensionPoints(PrefsPanel.class);
		for(IPluginExtensionPoint<PrefsPanel> prefExtPt:prefExtPts) {
			IPluginExtensionFactory<PrefsPanel> prefExtPtFactory = 
				prefExtPt.getFactory();
			
			PrefsPanel panel = prefExtPtFactory.createObject();
			
			tabbedPane.addTab(panel.getTitle(), panel);
		}
		
		header = new DialogHeader("Preferences", "Modify application preferences");
		
		setLayout(new BorderLayout());
		add(header, BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);
		
		PhonUIAction okAct = new PhonUIAction(this, "onOk");
		okAct.putValue(Action.NAME, "Ok");
		okAct.putValue(Action.SHORT_DESCRIPTION, "Save prefs and close dialog");
		
		okButton = new JButton(okAct);
		
		super.getRootPane().setDefaultButton(okButton);
		
		final ButtonBarBuilder barBuilder = new ButtonBarBuilder();
		JComponent buttonBar = barBuilder.addButton(okButton).build();
		
		add(buttonBar, BorderLayout.SOUTH);
	}
	
	public void setActiveTab(String tabName) {
		for(int tabIdx = 0; tabIdx < tabbedPane.getTabCount(); tabIdx++) {
			if(tabbedPane.getTitleAt(tabIdx).equals(tabName)) {
				tabbedPane.setSelectedIndex(tabIdx);
			}
		}
	}
	
	public void onOk(PhonActionEvent pae) {
		// save prefs and close dialog
		setVisible(false);
		dispose();
	}
	
	public static void main(String[] args) {
		PrefsDialog pd = new PrefsDialog();
		pd.pack();
		pd.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		pd.setModal(true);
		pd.setVisible(true);
		
		return;
	}

}
