package ca.phon.query.script;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import ca.phon.extensions.ExtensionSupport;
import ca.phon.extensions.IExtendable;
import ca.phon.project.Project;
import ca.phon.query.db.Query;
import ca.phon.query.db.QueryFactory;
import ca.phon.query.db.QueryManager;
import ca.phon.query.db.Script;
import ca.phon.util.PrefHelper;
import ca.phon.util.resources.FolderHandler;
import ca.phon.util.resources.ResourceLoader;

/**
 * Script library for phon query scripts.
 * 
 *
 */
public class QueryScriptLibrary implements IExtendable {

//	/**
//	 * Default script location
//	 */
//	public final static String SYSTEM_SCRIPT_FOLDER = "data/script";
//	
	/**
	 * User script folder
	 */
	public final static String USER_SCRIPT_FOLDER = 
			PrefHelper.getUserDataFolder() + File.separator + "script";
	
	private final ExtensionSupport extSupport = new ExtensionSupport(QueryScriptLibrary.class, this);
	
	private final ResourceLoader<QueryScript> systemScriptLoader = new ResourceLoader<QueryScript>();
	
	private final ResourceLoader<QueryScript> userScriptLoader = new ResourceLoader<QueryScript>();
	
	public static String projectScriptFolder(Project project) {
		return project.getLocation() + File.separator + "__res" + File.separator + "script";
	}
	
	/**
	 * Constructor
	 */
	public QueryScriptLibrary() {
		super();
		extSupport.initExtensions();
		
		initLoaders();
	}
	
	private void initLoaders() {
		final SystemQueryScriptHandler systemScriptHandler = new SystemQueryScriptHandler();
		systemScriptLoader.addHandler(systemScriptHandler);
		
		final UserFolderScriptHandler userFolderScriptHandler = new UserFolderScriptHandler(new File(USER_SCRIPT_FOLDER));
		userScriptLoader.addHandler(userFolderScriptHandler);
	}
	
	public ResourceLoader<QueryScript> stockScriptFiles() {
		return systemScriptLoader;
	}
	
	public ResourceLoader<QueryScript> userScriptFiles() {
		return userScriptLoader;
	}
	
	public ResourceLoader<QueryScript> projectScriptFiles(Project project) {
		final ResourceLoader<QueryScript> retVal = new ResourceLoader<QueryScript>();
		
		final UserFolderScriptHandler userFolderScriptHandler = new UserFolderScriptHandler(new File(projectScriptFolder(project)));
		retVal.addHandler(userFolderScriptHandler);
		
		return retVal;
	}
	
	/**
	 * Save given script to the specified file
	 * 
	 * @param script
	 * @param file
	 * 
	 * @throws IOException
	 */
	public static void saveScriptToFile(QueryScript script, String file) 
		throws IOException {
		if(file.endsWith(".js")) {
			BufferedWriter out = new BufferedWriter(
					new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
			out.write(script.getScript());
			out.flush();
			out.close();
		} else if(file.endsWith(".xml")) {
			final QueryScript qs = script;
			
			final QueryManager qm = QueryManager.getSharedInstance();
			final QueryFactory qf = qm.createQueryFactory();
			final Query q = qm.createQueryFactory().createQuery();
			
			q.setName((new File(file)).getName());
			final Script s = qf.createScript();
			s.setSource(qs.getScript());

			// TODO save params
			final Map<String, String> paramMap = new TreeMap<String, String>();
//			for(ScriptParam scriptParam:qs.getScriptParameters()) {
//				if(scriptParam.hasChanged()) {
//					for(String paramId:scriptParam.getParamIds()) {
//						final Object v = scriptParam.getValue(paramId);
//						if(v != null) {
//							paramMap.put(paramId, v.toString());
//						}
//					}
//				}
//			}
			s.setParameters(paramMap);
			q.setScript(s);
			
			qm.saveQuery(q, file);
		}
	}
	
	
	
	public Set<Class<?>> getExtensions() {
		return extSupport.getExtensions();
	}

	public <T> T getExtension(Class<T> cap) {
		return extSupport.getExtension(cap);
	}

	public <T> T putExtension(Class<T> cap, T impl) {
		return extSupport.putExtension(cap, impl);
	}

	public <T> T removeExtension(Class<T> cap) {
		return extSupport.removeExtension(cap);
	}

	
	
}
